Attribute VB_Name = "ModInfo"

'-- NOTE:   THIS MODULE IS ONLY COMMENTED DOCUMENTATION.
'--         IT WAS INCLUDED TO KEEP A COPY OF THIS INFO. WITH THE PROJECT.
' ___________________________________________________________________
'
'
''    _____________________________________________________
''    Accessing objects and information from open Explorer windows.
''    _____________________________________________________
''
''    -------------------------------------------------------------
''      Note: The code in this project is a working, disorganized rough draft. It is
''    functional but was only written as a test program for code used
''    in a class of an ActiveX DLL. You would probably want to edit
''    and/or restructure this code for your own purposes.
''    -------------------------------------------------------------
''
 '  In addition to this documentation, also see notes in code,
 '  including the 3 "gotchas" explained at the top.
''
''
''      The code here is rather eccentric and needs some explanation.
''    More to the point, the design of Microsoft's Shell objects and
''    interfaces is rather eccentric. It's amazing that there is so little
''    documentation, and so little usage, of the Shell objects. In general
''    they don 't seem to be well known.
''
''      This project got its start when someone asked, in a newsgroup,
''    how to return a list of selected items in an open folder. No one
''    seemed to know the answer. That led into research on how to
''    access the various shell objects and how to sort out their connections.
''
''       One of the resources I came across was a page on Raymond Chen's
''   weblog:
''        http://blogs.msdn.com/oldnewthing/archive/2004/07/20/188696.aspx
''
''      Raymond Chen is a Microsoft programmer who writes on various
''    programming issues. In this particular post he detailed how to get
''    into a folder's ShellFolderView using C++. It used numerous calls
''    to QueryInterface, involving about a half dozen different interfaces.
''    That was very challenging to convert to VB. Fortunately, as it turned
''    out, nearly all of Raymond Chen's code was superfluous and the way
''    to get into a folder using his general approach turned out to be quite
''    simple, though rather odd.
''
''      What follows is a general explanation of how this code works.
''    It starts out with a Shell.Windows enumeration, which may not be
''    familiar to most VB programmers. Shell.Windows is typically used
''    from scripting and normally has limited value. This code only gets
''    more weird from there! ... But it works, nevertheless.
''
''      ---- What does this code do? ----------------
''
''       It returns a ShellFolderView object and Shell Folder object for any
''    open (Web View) folder. That means it can get selected items, focused
''    item, etc.
''      On pre-XP systems it also returns the Document object for a Web View
''    folder, providing dynamic access to the DOM of the folder window
''    "viewport", which is actually a webpage in an IE browser window.
''
''    ------ How does it work? ----------------------
''
''       The arrangement of Windows Shell objects is fairly complex and
''    confusing. There are a number of redundant objects and interfaces
''    involved. There are also a number of odd, partial connections between
''    objects. The main reason for the mess seems to be the Active Desktop
''    package. So it might help to start there.... I apologize for the complexity of
''    the following explanation, but I can't think of any clearer way to detail the various
''    shell objects used in this code.
''
''     Active Desktop -
''
''       When Microsoft introduced Active Desktop with Web View folders, that
''    served as a move against Netscape by building IE into Windows. It
''    also served as a marketing tool, giving the impression that Microsoft was
''    somehow making Windows more Internet-connected at a time when the
''    Internet craze was at its height.
''
''        Web View folders and Active Desktop was mainly a gimmick. (After all, who
'      in their right mind would want a link to msn.com in their folder windows? ...or
'      a self-updating webpage embedded in their Desktop? ...or a Desktop billboard
'      with clickable links to go and view Disney ads?)
'         But Web View folders also represented a number of basic changes to the
'      Windows shell, and not all of those changes were entirely idiotic.
'
''     In brief, folder windows in Web View involve the following:
''
''      First, there are a number of confusing and only partially connected
''    interfaces, including IFolderView, IShellBrowser, IShellFolder, and IShellView.
''    Not to mention several different interfaces and objects that all represent
''    an IE window and have a partial connection to the Shell interfaces:
''    InternetExplorer, IWebBrowserApp, ShellBrowserWindow and WebBrowser.
''       Fortunately, none of that muddle needs to be dealt with here, except
''    IWebBrowserApp for accessing an IE instance.
''
''     XP does not actually have Web View folders -
''
''      Another thing to be aware of starting out: Web View folders apply to
''    Windows 95/NT4/98/2000/ME. Web View was dropped from Windows XP.
''    Although the general "Active Desktop / Web View" theme seems to continue
''    in XP, XP folder windows are just plain ListViews with added items
''    on the left side. They are not true Web View folders. They do not have an IE
''    browser window in their window hierarchy. (See Spy++.) However, XP does
''    have objects that emulate the Web View objects to maintain compatibility.
''    (More on that below.)
''
''    Shell objects -
''
''      A "Shell" object, and other related objects, were designed to encompass
''    much of what goes on in a Web View folder. Those objects were put into
''    shdoc401.dll with Active Desktop. On WinXP, there are matching objects
''    to emulate the same object model, in Shell32.dll.
''       The Shell objects are somehwat connected to Internet Explorer, in
''    shdocvw.dll.
''
''     The structure of a Web View folder -
''
''      In Web View, a folder window has an IE browser window two levels below
''    the top-level window. That browser window is the viewport of a folder
''    window. The folder view is actually a webpage. The visible file ListView is
''    actually an ActiveX control on that webpage. The page is folder.htt,
''    coming from the Windows\Web folder. That ActiveX control is not
''    different from the ShellFolderView object or the  hidden WebViewFolderContents
''    object in shdoc401.dll or shell32.dll. In the folder.htt webpage it
''    appears as an OBJECT named "FileList". ShellFolderView deals mainly with
''    actions in a folder window. It returns a SelectedItems collection, allows
''    for dynamic selection, etc. ShellFolderView also has a Folder property,
''    which returns a Folder object. The Folder object provides a collection of
''    items in a folder and provides information about those items. (Note, though,
'    that the Folder object is a Shell object. It returns what you see in a
'    folder window, not necessarily what's there. For instance, it will return
'     the size of a folder [Folder.Item.Size] as 0 bytes!)
''
''      The  shdoc401.dll / shell32.dll typelib also has a Shell object, which
''    has a Windows collection. The Shell.Windows collection is rather odd. It
''    returns a collection of open folder windows and running IE instances.
''    Each item is said to be an IE object. But it doesn't actually work that way.
''    The IE object returned for an open folder seems to be some kind of
''    wrapper object that partially emulates IE properties. The IE.hWnd
''    returned is actually the hWnd for the top-level window, while the Web View
''    browser window is two levels down. And the IE.Document returned is a dummy
''    object with no properties. On XP this arrangment is even more strange,
''    since there is no IE window in an XP folder window hierarchy!
''
'   Getting at the objects -
'
''      A Folder's Folder object can be returned via the Shell.Namespace function,
''    but the ShellFolderView object is normally only available from inside a
''    folder.htt page.
''
''      So the challenge is to get access to the folder and document objects from
''    an IE instance that's not IE, with a document property that's not a real
''    document...
''
''      The code here starts with a Shell.Windows collection and provides
''    three entirely different ways to get into the folder:
''
''    ---------------------------------
''    Method #1 : Using the IE.hWnd returned through Shell.Windows, go
''    down two levels and get the real browser window. From there, use
''    the obscure but documented function ObjectFromLresult in oleacc.dll
''    (Active Accessibility, which is installed on 2000/XP and can be installed
''    on Win98.) That function can return a Document object from an hWnd
''    if the window class is "Internet Explorer_Server". Using the DOM, get
''    the ShellFolderView object on the folder.htt webpage and get the Folder
'    object from that. Result: The actual Document, ShellFolderView and Folder
'   objects for an open folder are all returned and usable.
''      This method only works pre-XP because XP has no IE in folder
''    windows and thus no real document. However, this method can be used
''    to return a true IE document from a running IE instance on any system,
''    including XP.
''
''    --------------------------------
''
''    Method #2 : Get the IE.Document object through an IE instance
''    returned from Shell.Windows. (This is the dummy document, but it
'     still works in this usage.) Using QueryInterface, get the
''    ShellFolderView directly from that and thus get the Folder object
''    as well.
''      This method seems to work on all Windows versions. Even though
''    XP has no actual IE at in folder windows, much less an IE.Document, it
'     seems to emulate the Shell object model sufficiently for this method to work.
''
''     Note: This method cannot return the real Document object on any Window version.
''
''    ---------------------------------
''
''    Method # 3 : If for some reason #1 and #2 don't work, folder items and
''    selected items can be returned through Active Accessibility. (oleacc.dll)
''    In that case none of the Shell objects are available. Nor is the Document
''    object.
''
''    ---------------------------------
''
''     Methods #1 and #3 both require oleacc, but #2 should still work on all
''    systems. The only thing lacking from #2 is access to the Document
''    object of the folder window webpage.
''
''    -----------------------------------
'    Notes:
''   -----------------------------------
''
''    Using the DOM in folders or in IE -
''
''      While method #1 returns a "real" Document object for the folder window,
''    the functionality may be limited in some cases. Example: Using the normal
''    IE DOM, code was written to hide the file ListView in a selected window,
''    turn the background red, then write some test text in the window.
''    The code was as follows, tested via vbscript:
''        With oDoc
''               .All.Item("FileList").Style.display = "none"
''               .parentWindow.resizeTo 400, 700
''              .bgColor = "#FF0000"
''               .linkColor = "#00FF00"
''                .body.innerHTML = "<BR><FONT SIZE=5 COLOR=" & Chr(34) & "#FFFFFF" & Chr(34) & ">Shell Test.</FONT><BR>"
''          End With
''
''     The test code worked on WinME, but on Win98 with a custom folder.htt, only
''    the link color change and the window resize code seemed to work. It seems
''    that tables and frames pre-existing in the folder Web View may limit access.
''    Similarly, in a test of a running IE instance, code was able to entirely rewrite the
''    webpage in an about:blank window. But when the window held frames the
''    changes were not visible. Apparently the code was writing to the FRAMESET
''    page and therefore was not valid. The actual frames have separate
''    sub-windows in the IE window hierarchy. (Can be seen using Spy++.)
''
''    ----
''
''      Crashes when storing IEs -
''
''      This note is also in the code sample. The code stores the IE objects
''    returned from Shell.Windows in an array for easy access. On Win98
''    that caused Explorer to crash if all folder windows were closed
''    before releasing the object references. While the storage array is used
''    here for demo purposes, in the interest of stability an IE object should
''    probably never be stored. Better to re-iterate the Shell.Windows collection
''    when necessary and pass the IE object directly if needed for Method #2,
''    then pass it externally via a called function if desired, and release the
''    local copy.
''
''    ----
''
''       This project was tested on Win98SE, WinME and WinXP.
''    Method #1 worked on 98 and ME. Method #2 worked on XP.
'    Those were the expected results.
''    Presumably method #1 should work on Win2000, but that was
''    not tested.
''      A document object from a running instance of IE was returnable
''    in all tests - 98, ME, XP. (Although that specific code is not included here, it
''    only requires minor changes to the existing code in this project to
'     return an IE instance document rather than a folder window document.)
''    ----
'
'         The ListView control in a folder window, a SysListView32 class window,
'      is accessible with LVM_* messages, but only long returns will work.
'      That is, you can find, say, the number of selected items but not their names.
'      The reason is apparently because the buffer for the file names is not in
'      the same process and therefore no pointer variables are usable in the
'      SendMessage calls.
'
'     --------
'
'        After using the Folder object, returned from ShellFolderView.Folder,
''     to enumerate folder content, I realized that the Folder object is buggy
'      and actually hasd very little to offer. Enumerating folder content should
'      be done via API. See the EnumFileList sub in Mod1 for further explanation.
'
'
'

'''    -----------------------------------
''
''    Feel free to use this code as you see fit. You use it at your own risk.
''
''    Joe Priestley
''
''    jsware
''    www.jsware.net
''    jsware@ jsware.net
''
''
''
''
